function jointinfo = bvh_converter(FileName,DotNums,Subsample,Trim)
% This code reads in a bvh file and generates a .data3d.txt file. The
% .data3d.txt is the file that is the default input for the BioMotion
% class.
%
% Usage: bvhconverter(filename [,dotnum])
%
% FileName: give the full filename, including extension
%
% DotNums: the indices of the joints you want to scan
%
% Subsample: a target frame rate you want to acheive
%
% We tested it on several datasets. In some datesets the order of
% xyz is different, which will cause the action to be rotated when
% displayed with the default settings of the BioMotion class. This can be
% corrected with the .Rotation or .RotatePath() functionality in the
% BioMotion class, or with the dimchange argument in this function.
%
% JMB: added subsampling arg based on sample rate of original bvh file
%      added trim arg to allow for trimming off ends of segment
% HL: finalized version done on 8/20
% JvB: cleaned up, use dlmwrite(), and made such that dotnum is optional.

filename = [];
dotnum = [];
subsample = [];
trim = [0 0];

for argc = 0:nargin
    switch argc
        case 0
            % do nothing
        case 1
            if ~isempty(FileName) && ischar(FileName)
                filename = FileName;
            end
        case 2
            if ~isempty(DotNums)
                dotnum = DotNums;
            end
        case 3
            if ~isempty(Subsample) && isnumeric(Subsample)
                subsample = Subsample;
            end
        case 4
            if ~isempty(Trim) && isnumeric(Trim)
                trim = Trim;
            end
        otherwise
            % too many input args
    end
end

if isempty(filename)
    error('Need at least filename as input argument for bvh');
end

% read file, get recorded sample rate
[hd,motion,jointspos] = read_bvh(filename);
time_string = regexp(hd.motl2, '^Frame Time:\s*', 'end')+1;
bvh_samplerate = 1 / str2double(hd.motl2(time_string:end));
nframes = size(jointspos, 1);

if isempty(dotnum)
    dotnum = 1:(size(motion,2)/3-1);
end

if isempty(subsample)
    subsample = bvh_samplerate;
end

if subsample == bvh_samplerate
    % keep all frames
    frame_idx = 1:nframes;
else
    % discretize frames by timestamp
    time_sec = nframes / bvh_samplerate;
    bvh_timestamps = 0:(1/bvh_samplerate):time_sec;
    sub_timestamps = 0:(1/subsample):((ceil(time_sec * subsample) / subsample));
    new_timestamps = zeros(size(sub_timestamps));
    
    for i = 1:numel(sub_timestamps)
        t = sub_timestamps(i);
        idx = find(bvh_timestamps > t & bvh_timestamps <= t + (1/bvh_samplerate));
        if ~isempty(idx)
            new_timestamps(i) = idx;
        end
    end
    
    frame_idx = new_timestamps(new_timestamps > 0 & new_timestamps <= nframes);
end

% pull select frames based on sampling rate
jointspos = jointspos(frame_idx,dotnum,:);
nframes = size(jointspos, 1);

% trimming ends of sequence
if length(trim) ~= 2
    error('trim must be a vector of length 2')
end

if trim(1) == 0
    t_start = 1;
else
    t_start = trim(1) + 1;
end

if trim(2) == 0
    t_end = nframes;
else
    t_end = nframes - trim(2);
end

if t_start > t_end
    error('trim range is too large');
end

jointspos = jointspos(t_start:t_end,:,:);

% Save the data
a = permute(jointspos,[2 3 1]);
jointinfo = reshape(a,size(jointspos,2),size(jointspos,1)*3);

end


